@@ -396,9 +396,13 @@ class Pymata4(threading.Thread):
                         self.using_firmata_express = True
                         self.com_port = self.serial_port
                         return
+                    # continue trying
                     else:
-                        raise RuntimeError(f'arduino_instance_id does not match the '
-                                           f'value on the board.')
+                        continue
+
+            # did not find a match anywehre
+            raise RuntimeError(f'arduino_instance_id does not match '
+                                           f'a value on the boards.')
         except KeyboardInterrupt:
             raise RuntimeError('User Hit Control-C')
 
@@ -1540,6 +1544,8 @@ class Pymata4(threading.Thread):
 
             # retrieve previous value
             last_value = self.digital_pins[pin].current_value
+            if type(last_value) is list:
+                continue
 
             # set the current value in the pin structure
             self.digital_pins[pin].current_value = value
@@ -1726,26 +1732,26 @@ class Pymata4(threading.Thread):
 
         with self.the_sonar_map_lock:
             sonar_pin_entry = self.active_sonar_map[pin_number]
-            if sonar_pin_entry[0] is not None:
-                # check if value changed since last reading
-                if sonar_pin_entry[1] != val:
-                    sonar_pin_entry[1] = val
-                    time_stamp = time.time()
-                    sonar_pin_entry[2] = time_stamp
-                    self.active_sonar_map[pin_number] = sonar_pin_entry
-                    # Do a callback if one is specified in the table
-                    if sonar_pin_entry[0]:
-                        reply_data.append(pin_number)
-                        reply_data.append(val)
-                        reply_data.append(time_stamp)
-                        if sonar_pin_entry[1]:
-                            sonar_pin_entry[0](reply_data)
+            #if sonar_pin_entry[0] is not None:
+            # check if value changed since last reading
+            if sonar_pin_entry[1] != val:
+                sonar_pin_entry[1] = val
+                time_stamp = time.time()
+                sonar_pin_entry[2] = time_stamp
+                self.active_sonar_map[pin_number] = sonar_pin_entry
+                # Do a callback if one is specified in the table
+                if sonar_pin_entry[0]:
+                    reply_data.append(pin_number)
+                    reply_data.append(val)
+                    reply_data.append(time_stamp)
+                    if sonar_pin_entry[1]:
+                        sonar_pin_entry[0](reply_data)
 
-                # update the data in the table with latest value
-                else:
-                    sonar_pin_entry[1] = val
-                    self.active_sonar_map[pin_number] = sonar_pin_entry
-            time.sleep(self.sleep_tune)
+            # update the data in the table with latest value
+            else:
+                sonar_pin_entry[1] = val
+                self.active_sonar_map[pin_number] = sonar_pin_entry
+        time.sleep(self.sleep_tune)
 
     def _send_sysex(self, sysex_command, sysex_data=None):
         """
